@@ -19,8 +19,6 @@
 package org.apache.parquet.column.values.rle;
 
 import java.io.IOException;
-import java.nio.ByteBuffer;
-
 import org.apache.parquet.bytes.ByteBufferInputStream;
 import org.apache.parquet.column.values.ValuesReader;
 
@@ -31,16 +29,22 @@ import org.apache.parquet.column.values.ValuesReader;
  */
 public class ZeroIntegerValuesReader extends ValuesReader {
   
+  @Override
   public int readInteger() {
     return 0;
   }
 
   @Override
   public void initFromPage(int valueCount, ByteBufferInputStream stream) throws IOException {
+    updateNextOffset(0);
   }
 
   @Override
   public void skip() {
   }
 
+  @Override
+  public void skip(int n) {
+  }
+
 }
