@@ -43,7 +43,10 @@ public class ProtoParquetWriter<T extends MessageOrBuilder> extends ParquetWrite
    * @param blockSize            HDFS block size
    * @param pageSize             See parquet write up. Blocks are subdivided into pages for alignment and other purposes.
    * @throws IOException if there is an error while writing
+   *
+   * @deprecated will be removed in 2.0.0.; Use ProtoParquetWriter.Builder instead
    */
+  @Deprecated
   public ProtoParquetWriter(Path file, Class<? extends Message> protoMessage,
                             CompressionCodecName compressionCodecName, int blockSize,
                             int pageSize) throws IOException {
@@ -62,7 +65,10 @@ public class ProtoParquetWriter<T extends MessageOrBuilder> extends ParquetWrite
    * @param enableDictionary     Whether to use a dictionary to compress columns.
    * @param validating           to turn on validation using the schema
    * @throws IOException if there is an error while writing
+   *
+   * @deprecated will be removed in 2.0.0.; Use ProtoParquetWriter.Builder instead
    */
+  @Deprecated
   public ProtoParquetWriter(Path file, Class<? extends Message> protoMessage,
                             CompressionCodecName compressionCodecName, int blockSize,
                             int pageSize, boolean enableDictionary, boolean validating) throws IOException {
@@ -71,18 +77,20 @@ public class ProtoParquetWriter<T extends MessageOrBuilder> extends ParquetWrite
   }
 
   /**
-   * Create a new {@link ProtoParquetWriter}. The default block size is 50 MB.The default
-   * page size is 1 MB.  Default compression is no compression. (Inherited from {@link ParquetWriter})
+   * Create a new {@link ProtoParquetWriter}. The default block size is 128 MB. The default
+   * page size is 1 MB. Default compression is no compression. (Inherited from {@link ParquetWriter})
    *
    * @param file The file name to write to.
    * @param protoMessage         Protobuf message class
    * @throws IOException if there is an error while writing
+   *
+   * @deprecated will be removed in 2.0.0.; Use ProtoParquetWriter.Builder instead
    */
+  @Deprecated
   public ProtoParquetWriter(Path file, Class<? extends Message> protoMessage) throws IOException {
     this(file, protoMessage, CompressionCodecName.UNCOMPRESSED,
             DEFAULT_BLOCK_SIZE, DEFAULT_PAGE_SIZE);
   }
-  
   public static <T> Builder<T> builder(Path file) {
 	    return new Builder<T>(file);
 	}
@@ -90,13 +98,10 @@ public class ProtoParquetWriter<T extends MessageOrBuilder> extends ParquetWrite
 	public static <T> Builder<T> builder(OutputFile file) {
 	    return new Builder<T>(file);
 	}
-	
 	private static <T extends MessageOrBuilder> WriteSupport<T> writeSupport(Class<? extends Message> protoMessage) {
-		return new ProtoWriteSupport<T>(protoMessage);
+		return new ProtoWriteSupport<>(protoMessage);
 	}
-	  
 	public static class Builder<T> extends ParquetWriter.Builder<T, Builder<T>> {
-		  
 		Class<? extends Message> protoMessage = null;
 
 		private Builder(Path file) {
@@ -110,7 +115,6 @@ public class ProtoParquetWriter<T extends MessageOrBuilder> extends ParquetWrite
 		protected Builder<T> self() {
 		    return this;
 		}
-		
 		public Builder<T> withMessage(Class<? extends Message> protoMessage){
 			this.protoMessage = protoMessage;
 			return this;
@@ -118,6 +122,6 @@ public class ProtoParquetWriter<T extends MessageOrBuilder> extends ParquetWrite
 
 		protected WriteSupport<T> getWriteSupport(Configuration conf) {
 		    return (WriteSupport<T>) ProtoParquetWriter.writeSupport(protoMessage);
-		}    
+		}
 	}
 }
