@@ -18,8 +18,6 @@
  */
 package org.apache.parquet.thrift.projection.deprecated;
 
-import org.apache.hadoop.fs.GlobPattern;
-
 import com.google.re2j.Pattern;
 import com.google.re2j.PatternSyntaxException;
 
@@ -45,16 +43,6 @@ public class PathGlobPattern {
     set(globPattern);
   }
 
-  /**
-   * Compile glob pattern string
-   *
-   * @param globPattern the glob pattern
-   * @return the pattern object
-   */
-  public static Pattern compile(String globPattern) {
-    return new GlobPattern(globPattern).compiled();
-  }
-
   private static void error(String message, String pattern, int pos) {
     throw new PatternSyntaxException(String.format("%1s at %2d", message, pos), pattern);
   }
